var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g = Object.create((typeof Iterator === "function" ? Iterator : Object).prototype);
    return g.next = verb(0), g["throw"] = verb(1), g["return"] = verb(2), typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { useState, useCallback } from 'react';
/**
 * Custom hook for making HTTP requests using the Fetch API in React.
 * This hook provides methods for all standard HTTP methods and can handle
 * GET, POST, PUT, and DELETE requests.
 *
 * @template T - Type of the response data.
 * @template G - Type of the request body for methods like POST and PUT (optional).
 *
 * @param {string} url - The endpoint URL for the HTTP request.
 * @param {FetchOptions} options - Additional fetch options such as headers or credentials.
 *
 * @returns {FetchResponse<T, G>} - An object containing:
 *  - `data`: The response data from the request.
 *  - `error`: Any error message encountered during the request.
 *  - `loading`: A boolean indicating if the request is in progress.
 *  - `get`: A function to initiate a GET request.
 *  - `post`: A function to initiate a POST request with a request body.
 *  - `put`: A function to initiate a PUT request with a request body.
 *  - `delete`: A function to initiate a DELETE request.
 *
 * @example
 * ```typescript
 * const { data, error, loading, get, post } = useFetch<DataType>('https://api.example.com/data');
 *
 * useEffect(() => {
 *   get();
 * }, []);
 *
 * const handleSubmit = async (newData: RequestBodyType) => {
 *   await post(newData);
 * };
 * ```
 */
var useFetch = function (url, options) {
    if (options === void 0) { options = {}; }
    var _a = useState(null), data = _a[0], setData = _a[1];
    var _b = useState(null), error = _b[0], setError = _b[1];
    var _c = useState(false), loading = _c[0], setLoading = _c[1];
    var request = useCallback(function () {
        var args_1 = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args_1[_i] = arguments[_i];
        }
        return __awaiter(void 0, __spreadArray([], args_1, true), void 0, function (method, body) {
            var response, responseData, error_1;
            if (method === void 0) { method = 'GET'; }
            if (body === void 0) { body = null; }
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        setLoading(true);
                        setError(null);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 4, 5, 6]);
                        return [4 /*yield*/, fetch(url, __assign(__assign({}, options), { method: method, headers: __assign({ 'content-type': 'application/json' }, options.headers), body: body ? JSON.stringify(body) : null }))];
                    case 2:
                        response = _a.sent();
                        if (!response.ok) {
                            throw new Error("HTTP Error! Status: ".concat(response.status));
                        }
                        return [4 /*yield*/, response.json()];
                    case 3:
                        responseData = _a.sent();
                        setData(responseData);
                        return [3 /*break*/, 6];
                    case 4:
                        error_1 = _a.sent();
                        setError(error_1.message);
                        return [3 /*break*/, 6];
                    case 5:
                        setLoading(false);
                        return [7 /*endfinally*/];
                    case 6: return [2 /*return*/];
                }
            });
        });
    }, [url, options]);
    return {
        data: data,
        error: error,
        loading: loading,
        get: function () { return request('GET'); },
        post: function (body) { return request('POST', body); },
        put: function (body) { return request('PUT', body); },
        delete: function () { return request('DELETE'); },
    };
};
export default useFetch;
